window.Escapismal = window.Escapismal || {};
Escapismal['Actors'] = Escapismal['Actors'] || {};

Escapismal.Actors.add('tpotm_spiral_puzzle', (function () {

    let dec = '↊',
        el = '↋', rotateUp = 0, rotateDown = 0;

    let text = document.createElement('p');

    function formatDozenal(decimal) {
        let format = Number(decimal).toString(12).replace(/a/g, dec).replace(/b/g, el).replace(/\./g, ';');
        if (format.indexOf(';') === -1) {
            format += ';0';
        }
        format = format.substr(0, format.indexOf(';') + 2);
        return format;
    }

    function init(context, callback) {
        let gameState = context.gameState;
        let container = document.getElementById('obj_interactive_spiral');

        let angle = 137.5 - 15 * 2.25, curAngle = angle;
        const width = window.innerWidth;
        const height = window.innerHeight;
        const NS = "http://www.w3.org/2000/svg";
        const svg = document.createElementNS(NS, "svg");
        svg.setAttribute("viewBox", `0 0 ${width} ${height}`)
        svg.setAttribute("width", `${width}px`)
        svg.setAttribute("height", `${height}px`)
        svg.setAttribute("xmlns", NS);
        container.appendChild(svg);
        const center = [width * 0.5, height * 0.5]
        let n = 0;
        const c = 10;
        let seedAngle = Math.PI / 180 * angle;
        for (n = 0; n < 1000; n++) {
            const dot = document.createElementNS(NS, "circle");
            dot.setAttribute('r', 4);
            dot.setAttribute('fill', '#aaa');
            dot.setAttribute('stroke', '#999');

            const a = n * curAngle * Math.PI / 180;
            const r = c * Math.pow(n, 0.5);
            dot.setAttribute('cx', center[0] + r * Math.cos(a));
            dot.setAttribute('cy', center[1] + r * Math.sin(a));
            svg.appendChild(dot)
        }

        document.getElementById('obj_frame_text').innerHTML = generateDrawnString(formatDozenal(angle) + '°');

        function reset() {
            angle = 137.5 - 15 * 2.25;
            curAngle = angle;
            rotateUp = 0;
            rotateDown = 0;

            document.getElementById('obj_frame_knob_down').style.transform = 'rotate(' + rotateDown + 'deg)';
            document.getElementById('obj_frame_knob_up').style.transform = 'rotate(' + rotateUp + 'deg)';
            document.getElementById('obj_frame_text').innerHTML = generateDrawnString(formatDozenal(angle) + '°');

        }

        function knobUpHandler(e) {
            angle += 2.25;
            rotateUp += 120;
            document.getElementById('obj_frame_knob_up').style.transform = 'rotate(' + rotateUp + 'deg)';
            document.getElementById('obj_frame_text').innerHTML = generateDrawnString(formatDozenal(angle) + '°');
            context.commandHandler.execute([{type:'playSound', payload:{id: 'sfx_click'}}]);
        }

        function knobDownHandler(e) {
            angle -= 2.25;
            rotateDown -= 120;
            document.getElementById('obj_frame_knob_down').style.transform = 'rotate(' + rotateDown + 'deg)';
            document.getElementById('obj_frame_text').innerHTML = generateDrawnString(formatDozenal(angle) + '°');
            context.commandHandler.execute([{type:'playSound', payload:{id: 'sfx_click'}}]);
        }

        let upHandlers = Escapismal.Input.addActionListener(document.getElementById('obj_frame_knob_up'), knobUpHandler);
        let downHandlers = Escapismal.Input.addActionListener(document.getElementById('obj_frame_knob_down'), knobDownHandler);

        let interval = window.setInterval(function () {
            if (Math.round(curAngle * 10) === Math.round(angle * 10)) {
                if (Math.abs(curAngle - 137.5) < 0.01) {
                    context.eventHandler.process([{type: 'tpotm_spiral_puzzle_ended', payload:{}}]);
                    if (callback) callback();
                }
                return;
            }
            if (curAngle < angle) {
                curAngle += 1. / 12;
            } else if (curAngle > angle) {
                curAngle -= 1. / 12;
            }

            [...svg.querySelectorAll('circle')].forEach(function (dot, n) {
                const a = n * curAngle * Math.PI / 180;
                const r = c * Math.pow(n, 0.5);
                dot.setAttribute('cx', center[0] + r * Math.cos(a));
                dot.setAttribute('cy', center[1] + r * Math.sin(a));

            });
        }, 20);

        function end(){
            document.getElementById('obj_frame_text').innerHTML = generateDrawnString(formatDozenal(137.5) + '°');
            Escapismal.Input.removeActionListener(document.getElementById('obj_frame_knob_up'), upHandlers);
            Escapismal.Input.removeActionListener(document.getElementById('obj_frame_knob_down'), downHandlers);
            window.clearInterval(interval);
            context.eventHandler.removeListener(eventListener);
        }

        let eventListener = {
            process: function(event){
                if (event.type ==='tpotm_spiral_puzzle_ended') {
                    end();
                    return true;
                } else if (event.type === 'navigateTo' && event.payload === 'loc_hall_frame'){
                    reset();
                }
                return false;
            }
        };

        context.eventHandler.addListener(eventListener);
    }

    return {
        init: init
    }
})());
